@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2021, OFFIS e.V.
+ *  Copyright (C) 1994-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -138,7 +138,7 @@ stripWhitespace(char *s)
 
     p = s;
     while (*s != '\0') {
-        if (isspace(TO_UCHAR(*s)) == OFFalse) {
+        if (OFStandard::isspace(*s) == OFFalse) {
             *p++ = *s;
         }
         s++;
@@ -150,7 +150,7 @@ static char *
 stripTrailingWhitespace(char *s)
 {
     if (s == NULL) return s;
-    for (size_t i = strlen(s); i > 0 && isspace(TO_UCHAR(s[--i])); s[i] = '\0');
+    for (size_t i = strlen(s); i > 0 && OFStandard::isspace(s[--i]); s[i] = '\0');
     return s;
 }
 
@@ -161,7 +161,7 @@ stripPrecedingWhitespace(char *s)
     char *p;
     if (s == NULL) return s;
 
-    for(p = s; *p && isspace(TO_UCHAR(*p)); p++)
+    for(p = s; *p && OFStandard::isspace(*p); p++)
         ;
 
     return p;
@@ -170,7 +170,7 @@ stripPrecedingWhitespace(char *s)
 static OFBool
 onlyWhitespace(const char *s)
 {
-    while(*s) if (!isspace(TO_UCHAR(*s++)))
+    while(*s) if (!OFStandard::isspace(*s++))
         return OFFalse;
     return OFTrue;
 }
@@ -201,7 +201,7 @@ static OFBool
 isaCommentLine(const char *s)
 {
     // skip leading spaces
-    while(isspace(TO_UCHAR(*s))) ++s;
+    while(OFStandard::isspace(*s)) ++s;
     return *s == DCM_DumpCommentChar;
 }
 
@@ -626,10 +626,10 @@ insertIntoSet(DcmStack &stack, const E_TransferSyntax xfer, const DcmTagKey &tag
         }
         else if (newElementError == EC_InvalidTag)
         {
-            if (tag.getXTag() == DCM_Item)
+            if (tag == DCM_Item)
             {
                 DcmItem *item = NULL;
-                if (topOfStack->getTag().getXTag() == DCM_DirectoryRecordSequence)
+                if (topOfStack->getTag() == DCM_DirectoryRecordSequence)
                 {
                     // an Item must be pushed to the stack
                     item = new DcmDirectoryRecord(tag, 0);
@@ -792,7 +792,7 @@ int main(int argc, char *argv[])
     cmd.setParamColumn(LONGCOL + SHORTCOL + 4);
 
     cmd.addParam("dumpfile-in", "dump input filename");
-    cmd.addParam("dcmfile-out", "DICOM output filename");
+    cmd.addParam("dcmfile-out", "DICOM output filename (\"-\" for stdout)");
 
     cmd.addGroup("general options:", LONGCOL, SHORTCOL + 2);
      cmd.addOption("--help",                   "-h",     "print this help text and exit", OFCommandLine::AF_Exclusive);
@@ -1012,7 +1012,7 @@ int main(int argc, char *argv[])
     }
     OFLOG_INFO(dump2dcmLogger, "reading dump file: " << opt_ifname);
 
-    FILE *dumpfile = fopen(opt_ifname, "r");
+    FILE *dumpfile = fopen(opt_ifname, "rb");
     if (!dumpfile)
     {
         OFLOG_FATAL(dump2dcmLogger, "input file does not exist: " << opt_ifname);
@@ -1067,7 +1067,7 @@ int main(int argc, char *argv[])
         if (fileformat.canWriteXfer(opt_xfer))
         {
             /* check whether pixel data is compressed */
-            if ((opt_writeMode == EWM_dataset) && DcmXfer(xfer).isEncapsulated())
+            if ((opt_writeMode == EWM_dataset) && DcmXfer(xfer).usesEncapsulatedFormat())
             {
                 OFLOG_WARN(dump2dcmLogger, "encapsulated pixel data require file format, ignoring --write-dataset");
                 opt_writeMode = EWM_fileformat;
