@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2021, OFFIS e.V.
+ *  Copyright (C) 1994-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -95,9 +95,9 @@ class DCMTK_DCMDATA_EXPORT DcmItem
      *          cannot be casted to this object type or both objects are of
      *          different VR (i.e. the DcmEVR returned by the element's ident()
      *          call are different).
-     *          1 if either this element has more components than the rhs element, or
-     *          if the first component that does not match is greater in this object than
-     *          in rhs object.
+     *          1 if either this element has more components than the rhs
+     *          element, or if the first component that does not match is
+     *          greater in this object than in rhs object.
      */
     virtual int compare(const DcmItem& rhs) const;
 
@@ -448,8 +448,8 @@ class DCMTK_DCMDATA_EXPORT DcmItem
      *  that is, DcmItem and DcmSequenceOfItems. It returns a pointer to the
      *  next object in the list AFTER the given object. If the caller passes NULL,
      *  a pointer to the first object in the list is returned. If the given object
-     *  is not found, the given object is the last one in the list or the list is empty,
-     *  NULL is returned.
+     *  is not found, the given object is the last one in the list or the list is
+     *  empty, NULL is returned.
      *  @param obj pointer to one object in the container; we are looking for the
      *    next entry after this one. NULL if looking for the first entry.
      *  @return pointer to next object in container or NULL if not found
@@ -1206,6 +1206,19 @@ class DCMTK_DCMDATA_EXPORT DcmItem
                                    const unsigned long pos = 0,
                                    const OFBool replaceOld = OFTrue);
 
+    /** create a new element, put specified value to it and insert the element into the dataset/item.
+     *  Applicable to the following VRs: SL.
+     *  @param tag DICOM tag specifying the attribute to be created
+     *  @param value value to be set for the new element
+     *  @param count number of values (not bytes!) to be copied from 'value'
+     *  @param replaceOld flag indicating whether to replace an existing element or not
+     *  @return EC_Normal upon success, an error code otherwise.
+     */
+    OFCondition putAndInsertSint32Array(const DcmTag &tag,
+                                        const Sint32 *value,
+                                        const unsigned long count,
+                                        const OFBool replaceOld = OFTrue);
+
     /** create a new element, put specified value to it and insert the element into the dataset/item.
      *  Applicable to the following VRs: FL, OF.
      *  @param tag DICOM tag specifying the attribute to be created
@@ -1308,8 +1321,8 @@ class DCMTK_DCMDATA_EXPORT DcmItem
      *  in the dictionary) will result in a DcmElement instance of derived type
      *  DcmOtherByteOtherWord.
      *  @param tag attribute tag of the element to be created
-     *  @param privateCreator private creator of the element, if element tag
-     *    is private (default: NULL, i.e. non-private DICOM standard tag)
+     *  @param privateCreator private creator identifier of the element, if element
+     *    tag is private (default: NULL, i.e. non-private DICOM standard tag)
      *  @return pointer to newly created element upon success, NULL pointer otherwise
      *
      */
@@ -1323,8 +1336,8 @@ class DCMTK_DCMDATA_EXPORT DcmItem
      *  @param newElement pointer to newly created element returned in this parameter
      *    upon success, NULL pointer otherwise
      *  @param tag attribute tag of the element to be created
-     *  @param privateCreator private creator of the element, if element tag
-     *    is private (default: NULL, i.e. non-private DICOM standard tag)
+     *  @param privateCreator private creator identifier of the element, if element
+     *    tag is private (default: NULL, i.e. non-private DICOM standard tag)
      *  @return EC_Normal upon success, an error code otherwise
      */
     static OFCondition newDicomElement(DcmElement *&newElement,
@@ -1425,9 +1438,10 @@ class DCMTK_DCMDATA_EXPORT DcmItem
      *  'US' depending on the value of PixelRepresentation.
      *  @param item dataset or item that can be used to lookup other element values
      *  @param tag tag of the element to be checked and updated (if required)
+     *  @return OFTrue if the VR has changed, OFFalse otherwise
      */
-    void checkAndUpdateVR(DcmItem &item,
-                          DcmTag &tag);
+    OFBool checkAndUpdateVR(DcmItem &item,
+                            DcmTag &tag);
 
     /** update the SpecificCharacterSet (0008,0005) element depending on the given
      *  parameters. The current value of this element is either replaced or a new
@@ -1451,8 +1465,8 @@ class DCMTK_DCMDATA_EXPORT DcmItem
      *  @param tag attribute tag of the element to be created. VR of tag may be
      *    updated within the method.
      *  @param length attribute value length of the element to be created
-     *  @param privateCreatorCache cache object for private creator strings in
-     *    the current dataset
+     *  @param privateCreatorCache cache object for private creator elements
+     *    in the current dataset
      *  @param readAsUN flag indicating whether parser is currently handling
      *    UN element that must be read in implicit VR little endian; updated
      *    upon return
@@ -1487,7 +1501,7 @@ class DCMTK_DCMDATA_EXPORT DcmItem
      */
     static OFBool foundVR(const Uint8* atposition);
 
-    /// cache for private creator tags and names
+    /// cache for private creator tags and identifiers
     DcmPrivateTagCache privateCreatorCache;
 };
 
