@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2021, OFFIS e.V.
+ *  Copyright (C) 1994-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -22,14 +22,18 @@
 
 #include "dcmtk/config/osconfig.h"    /* make sure OS specific configuration is included first */
 
-#include "dcmtk/ofstd/ofstd.h"
 #include "dcmtk/dcmdata/dcdict.h"
-#include "dcmtk/ofstd/ofdefine.h"
 #include "dcmtk/dcmdata/dcdicent.h"
 #include "dcmtk/dcmdata/dctypes.h"
+#include "dcmtk/ofstd/ofdefine.h"
 #include "dcmtk/ofstd/ofstd.h"
 #include "dcmtk/ofstd/offile.h"
 
+#ifdef HAVE_WINDOWS_H
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#endif
+
 /*
 ** The separator character between fields in the data dictionary file(s)
 */
@@ -138,7 +142,7 @@ stripWhitespace(char* s)
     unsigned char *t;
     unsigned char *p;
     t=p=OFreinterpret_cast(unsigned char *, s);
-    while ((c = *t++) != '\0') if (!isspace(c)) *p++ = c;
+    while ((c = *t++) != '\0') if (!OFStandard::isspace(c)) *p++ = c;
     *p = '\0';
   }
 }
@@ -150,7 +154,7 @@ stripTrailingWhitespace(char* s)
     for
     (
         char* it = s + strlen(s) - 1;
-        it >= s && isspace(OFstatic_cast(unsigned char, *it));
+        it >= s && OFStandard::isspace(*it);
         *it-- = '\0'
     );
     return s;
@@ -165,7 +169,7 @@ stripLeadingWhitespace(char* s)
     unsigned char *t;
     unsigned char *p;
     t=p=OFreinterpret_cast(unsigned char *, s);
-    while (isspace(*t)) t++;
+    while (OFStandard::isspace(*t)) t++;
     while ((c = *t++) != '\0') *p++ = c;
     *p = '\0';
   }
@@ -356,7 +360,7 @@ parseWholeTagField(char* s, DcmTagKey& key,
 
     if (pi > 0)
     {
-      // copy private creator name
+      // copy private creator identifier
       size_t buflen = strlen(pc) + 1;
       privCreator = new char[buflen]; // deleted by caller
       if (privCreator) OFStandard::strlcpy(privCreator, pc, buflen);
@@ -375,7 +379,7 @@ onlyWhitespace(const char* s)
     int charsFound = OFFalse;
 
     for (size_t i = 0; (!charsFound) && (i < len); ++i) {
-        charsFound = !isspace(OFstatic_cast(unsigned char, s[i]));
+        charsFound = !OFStandard::isspace(s[i]);
     }
     return (!charsFound)? (OFTrue) : (OFFalse);
 }
@@ -399,7 +403,7 @@ isaCommentLine(const char* s)
     OFBool isComment = OFFalse; /* assumption */
     size_t len = strlen(s);
     size_t i = 0;
-    for (i = 0; i < len && isspace(OFstatic_cast(unsigned char, s[i])); ++i) /*loop*/;
+    for (i = 0; i < len && OFStandard::isspace(s[i]); ++i) /*loop*/;
     isComment = (s[i] == DCM_DICT_COMMENT_CHAR);
     return isComment;
 }
@@ -571,15 +575,6 @@ DcmDataDictionary::loadDictionary(const char* fileName, OFBool errorIfAbsent)
     }
 }
 
-#ifndef HAVE_GETENV
-
-static
-char* getenv() {
-    return NULL;
-}
-
-#endif /* !HAVE_GETENV */
-
 
 
 OFBool
@@ -604,6 +599,10 @@ DcmDataDictionary::loadExternalDictionaries()
 #endif
     }
 
+#ifdef HAVE_WINDOWS_H
+    char buf[MAX_PATH+1];
+#endif
+
     /* if any mechanism for external dictionary (environment or default external)
      * is actually provided it, parse env and load all dictionaries specified therein.
      */
@@ -616,6 +615,11 @@ DcmDataDictionary::loadExternalDictionaries()
         }
 
         if (sepCnt == 0) {
+#ifdef HAVE_WINDOWS_H
+            memset(buf, 0, sizeof(buf));
+            (void) ExpandEnvironmentStringsA(env, buf, sizeof(buf));
+            env = buf;
+#endif
             if (!loadDictionary(env, OFTrue)) {
                 return OFFalse;
             }
@@ -629,9 +633,18 @@ DcmDataDictionary::loadExternalDictionaries()
 
             for (int ii = 0; ii < ndicts; ii++) {
                 if ((dictArray[ii] != NULL) && (strlen(dictArray[ii]) > 0)) {
+#ifdef HAVE_WINDOWS_H
+                    memset(buf, 0, sizeof(buf));
+                    (void) ExpandEnvironmentStringsA(dictArray[ii], buf, sizeof(buf));
+                    env = buf;
+                    if (!loadDictionary(buf, OFTrue)) {
+                        loadFailed = OFTrue;
+                    }
+#else
                     if (!loadDictionary(dictArray[ii], OFTrue)) {
                         loadFailed = OFTrue;
                     }
+#endif
                 }
                 free(dictArray[ii]);
             }
@@ -648,7 +661,7 @@ DcmDataDictionary::addEntry(DcmDictEntry* e)
 {
     if (e->isRepeating()) {
         /*
-         * Find the best position in repeating tag list
+         * Find the best position in repeating groups data dictionary.
          * Existing entries are replaced if the ranges and repetition
          * constraints are the same.
          * If a range represents a subset of an existing range then it
@@ -725,14 +738,13 @@ DcmDataDictionary::findEntry(const DcmDictEntry& entry) const
 const DcmDictEntry*
 DcmDataDictionary::findEntry(const DcmTagKey& key, const char *privCreator) const
 {
-    /* search first in the normal tags dictionary and if not found
-     * then search in the repeating tags list.
+    /* first, search in the normal tags data dictionary (including private tags)
      */
     const DcmDictEntry* e = NULL;
-
     e = hashDict.get(key, privCreator);
+
     if (e == NULL) {
-        /* search in the repeating tags dictionary */
+        /* then, search in the repeating groups data dictionary */
         OFBool found = OFFalse;
         DcmDictEntryListConstIterator iter(repDict.begin());
         DcmDictEntryListConstIterator last(repDict.end());
@@ -752,8 +764,7 @@ DcmDataDictionary::findEntry(const char *name) const
     const DcmDictEntry* e = NULL;
     const DcmDictEntry* ePrivate = NULL;
 
-    /* search first in the normal tags dictionary and if not found
-     * then search in the repeating tags list.
+    /* first, search in the normal tags data dictionary
      */
     DcmHashDictIterator iter;
     for (iter = hashDict.begin(); (e == NULL) && (iter != hashDict.end()); ++iter) {
@@ -768,8 +779,9 @@ DcmDataDictionary::findEntry(const char *name) const
         }
     }
 
-    if (e == NULL) {
-        /* search in the repeating tags dictionary */
+    /* if not found ... */
+     if (e == NULL) {
+        /* search in the repeating groups data dictionary */
         OFBool found = OFFalse;
         DcmDictEntryListConstIterator iter2(repDict.begin());
         DcmDictEntryListConstIterator last(repDict.end());
