@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2008-2021, OFFIS e.V.
+ *  Copyright (C) 2008-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -21,8 +21,8 @@
  */
 
 #include "dcmtk/config/osconfig.h" /* make sure OS specific configuration is included first */
-
 #include "dcmtk/dcmdata/dcpath.h"
+#include "dcmtk/ofstd/ofstd.h"
 #include "dcmtk/dcmdata/dcsequen.h"
 
 /*******************************************************************/
@@ -120,11 +120,11 @@ OFString DcmPath::toString() const
         else if ((vr == EVR_item) || (vr == EVR_dataset))
         {
 #ifdef PRIu32
-            sprintf(buf, "[%" PRIu32 "]", (*it)->m_itemNo);
+            OFStandard::snprintf(buf, sizeof(buf), "[%" PRIu32 "]", (*it)->m_itemNo);
 #elif SIZEOF_LONG == 8
-            sprintf(buf, "[%u]", (*it)->m_itemNo);
+            OFStandard::snprintf(buf, sizeof(buf), "[%u]", (*it)->m_itemNo);
 #else
-            sprintf(buf, "[%lu]", (*it)->m_itemNo);
+            OFStandard::snprintf(buf, sizeof(buf), "[%lu]", (*it)->m_itemNo);
 #endif
             pathStr.append(buf);
             it++;
@@ -157,6 +157,23 @@ OFBool DcmPath::containsGroup(const Uint16 groupNo) const
     return OFFalse;
 }
 
+// Checks whether an invalid group number is used in the path's path nodes
+OFBool DcmPath::containsInvalidGroup() const
+{
+    OFListConstIterator(DcmPathNode*) it        = m_path.begin();
+    OFListConstIterator(DcmPathNode*) endOfList = m_path.end();
+    while (it != endOfList)
+    {
+        DcmPathNode* node = *it;
+        if ((node == NULL) || (node->m_obj == NULL))
+            return OFFalse;
+        if (!node->m_obj->getTag().getTagKey().hasValidGroup())
+            return OFTrue;
+        it++;
+    }
+    return OFFalse;
+}
+
 // Helper function for findOrCreatePath(). Parses item no from start of string
 OFCondition DcmPath::parseItemNoFromPath(OFString& path,      // inout
                                          Uint32& itemNo,      // out
@@ -291,13 +308,13 @@ OFCondition DcmPath::separatePathNodes(const OFString& path, OFList<OFString>& r
             else
             {
 #ifdef PRIu32
-                if (sprintf(buf, "[%" PRIu32 "]", itemNo) < 2)
+                if (OFStandard::snprintf(buf, sizeof(buf), "[%" PRIu32 "]", itemNo) < 2)
                     return EC_IllegalParameter;
 #elif SIZEOF_LONG == 8
-                if (sprintf(buf, "[%u]", itemNo) < 2)
+                if (OFStandard::snprintf(buf, sizeof(buf), "[%u]", itemNo) < 2)
                     return EC_IllegalParameter;
 #else
-                if (sprintf(buf, "[%lu]", itemNo) < 2)
+                if (OFStandard::snprintf(buf, sizeof(buf), "[%lu]", itemNo) < 2)
                     return EC_IllegalParameter;
 #endif
                 result.push_back(buf);
@@ -310,7 +327,7 @@ OFCondition DcmPath::separatePathNodes(const OFString& path, OFList<OFString>& r
             status = parseTagFromPath(pathStr, tag);
             if (status.bad())
                 return status;
-            if (sprintf(buf, "(%04X,%04X)", tag.getGroup(), tag.getElement()) != 11)
+            if (OFStandard::snprintf(buf, sizeof(buf), "(%04X,%04X)", tag.getGroup(), tag.getElement()) != 11)
                 return EC_IllegalParameter;
             result.push_back(buf);
             nextIsItem = OFTrue;
@@ -650,7 +667,7 @@ OFCondition DcmPathProcessor::findOrCreateItemPath(DcmItem* item, OFString& path
                     // check private reservation if desired
                     if (m_checkPrivateReservations)
                     {
-                        status = checkPrivateTagReservation(item, tag.getXTag());
+                        status = checkPrivateTagReservation(item, tag.getTagKey());
                         if (status.bad())
                             return status;
                     }
@@ -942,11 +959,11 @@ DcmPathProcessor::checkPrivateTagReservation(DcmItem* item, const DcmTagKey& tag
         // check whether private creator is correct
         if (actualPrivateCreator != privateCreator)
         {
-            OFString msg = "Private creator string (";
+            OFString msg = "Private creator identifier (";
             msg += actualPrivateCreator;
             msg += ") other than expected ( ";
             msg += privateCreator;
-            msg += privateCreator;
+            msg += ")";
             return makeOFCondition(OFM_dcmdata, 25, OF_error, msg.c_str());
         }
     }
