@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2021, OFFIS e.V.
+ *  Copyright (C) 1994-2023, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -549,16 +549,16 @@ OFCondition DcmSequenceOfItems::makeSubObject(DcmObject *&subObject,
     switch (newTag.getEVR())
     {
         case EVR_na:
-            if (newTag.getXTag() == DCM_Item)
+            if (newTag == DCM_Item)
             {
-                if (getTag().getXTag() == DCM_DirectoryRecordSequence)
+                if (getTag() == DCM_DirectoryRecordSequence)
                     subItem = new DcmDirectoryRecord(newTag, newLength);
                 else
                     subItem = new DcmItem(newTag, newLength);
             }
-            else if (newTag.getXTag() == DCM_SequenceDelimitationItem)
+            else if (newTag == DCM_SequenceDelimitationItem)
                 l_error = EC_SequEnd;
-            else if (newTag.getXTag() == DCM_ItemDelimitationItem)
+            else if (newTag == DCM_ItemDelimitationItem)
                 l_error = EC_ItemEnd;
             else
                 l_error = EC_InvalidTag;
@@ -796,6 +796,10 @@ OFCondition DcmSequenceOfItems::write(DcmOutputStream &outStream,
         {
             if (getTransferState() == ERW_init)
             {
+                // Force a compression filter (if any) to process the input buffer, by calling outStream.write().
+                // This ensures that we cannot get stuck if there are just a few bytes available in the buffer
+                outStream.write(NULL, 0);
+
                 /* first compare with DCM_TagInfoLength (12). If there is not enough space
                  * in the buffer, check if the buffer is still sufficient for the requirements
                  * of this element, which may need only 8 instead of 12 bytes.
@@ -838,6 +842,9 @@ OFCondition DcmSequenceOfItems::write(DcmOutputStream &outStream,
                     setTransferState(ERW_ready);
                     if (getLengthField() == DCM_UndefinedLength)
                     {
+                        // Force a compression filter (if any) to process the input buffer, by calling outStream.write().
+                        // This ensures that we cannot get stuck if there are just a few bytes available in the buffer
+                        outStream.write(NULL, 0);
                         if (outStream.avail() >= 8)
                         {
                             // write sequence delimitation item
@@ -920,6 +927,10 @@ OFCondition DcmSequenceOfItems::writeSignatureFormat(DcmOutputStream &outStream,
         {
             if (getTransferState() == ERW_init)
             {
+                // Force a compression filter (if any) to process the input buffer, by calling outStream.write().
+                // This ensures that we cannot get stuck if there are just a few bytes available in the buffer
+                outStream.write(NULL, 0);
+
                 /* first compare with DCM_TagInfoLength (12). If there is not enough space
                  * in the buffer, check if the buffer is still sufficient for the requirements
                  * of this element, which may need only 8 instead of 12 bytes.
@@ -957,6 +968,11 @@ OFCondition DcmSequenceOfItems::writeSignatureFormat(DcmOutputStream &outStream,
                 if (errorFlag.good())
                 {
                     setTransferState(ERW_ready);
+
+                    // Force a compression filter (if any) to process the input buffer, by calling outStream.write().
+                    // This ensures that we cannot get stuck if there are just a few bytes available in the buffer
+                    outStream.write(NULL, 0);
+
                     /* we always write a sequence delimitation item tag, but no length */
                     if (outStream.avail() >= 4)
                     {
