@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2019-2021, OFFIS e.V.
+ *  Copyright (C) 2019-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -22,7 +22,6 @@
 #include "dcmtk/config/osconfig.h" /* make sure OS specific configuration is included first */
 
 #include "dcmtk/ofstd/ofmem.h"
-#include "dcmtk/ofstd/ofstrutl.h"
 #include "dcmtk/ofstd/oftempf.h"
 #include "dcmtk/ofstd/oftest.h"
 
@@ -30,7 +29,9 @@
 
 #include "dcmtk/dcmdata/dcxfer.h"
 #include "dcmtk/dcmdata/dcswap.h"
-
+#include "dcmtk/dcmdata/dcfilefo.h"
+#include "dcmtk/dcmdata/dcdatset.h"
+#include "dcmtk/dcmdata/dcdict.h"
 #include "dcmtk/dcmfg/concatenationcreator.h"
 #include "dcmtk/dcmfg/concatenationloader.h"
 #include "dcmtk/dcmfg/fgctacquisitiondetails.h"
@@ -51,10 +52,9 @@
 #include "dcmtk/dcmfg/fgpixmsr.h"
 #include "dcmtk/dcmfg/fgplanor.h"
 #include "dcmtk/dcmfg/fgplanpo.h"
-#include "dcmtk/dcmfg/fgrealworldvaluemapping.h"
 #include "dcmtk/dcmfg/fgtemporalposition.h"
 
-static OFLogger tRoundLogger = OFLog::getLogger("dcmtk.test.t_roundtrip");
+static OFLogger tRoundLogger = OFLog::getLogger("dcmtk.test.troundtrip");
 
 // Do not change values below since
 // a) The expected dataset (dump) is made for these values
@@ -62,7 +62,7 @@ static OFLogger tRoundLogger = OFLog::getLogger("dcmtk.test.t_roundtrip");
 //    4 GB, otherwise the calls to writeDataset() will fail.
 // The test dcmect/tests/t_huge_concat.cc allows for exercising
 // "unlimited" pixel data size using writeConcatenation() on all
-// occassions.
+// occasions.
 
 static const Uint16 NUM_ROWS             = 2;
 static const Uint16 NUM_COLS             = 2;
@@ -500,13 +500,11 @@ static void checkConcatenationInstance(size_t numInstance, EctEnhancedCT* srcIns
 
         FunctionalGroups::const_iterator srcShared = srcInstance->getFunctionalGroups().getShared()->begin();
         FunctionalGroups::const_iterator cShared   = concat->getFunctionalGroups().getShared()->begin();
-        size_t numShared                           = 0;
         do
         {
             OFCHECK(srcShared->second->compare(*cShared->second) == 0);
             srcShared++;
             cShared++;
-            numShared++;
         } while ((srcShared != srcInstance->getFunctionalGroups().getShared()->end())
                  && (cShared != concat->getFunctionalGroups().getShared()->end()));
         OFCHECK((srcShared == srcInstance->getFunctionalGroups().getShared()->end())
@@ -662,7 +660,7 @@ static void prepareExpectedDump()
 {
     EXPECTED_DUMP += "\n";
     EXPECTED_DUMP += "# Dicom-Data-Set\n";
-    // DcmDataset.print() produces dumps in local endianess, so make sure the dump reflects the current machine
+    // DcmDataset.print() produces dumps in local endianness, so make sure the dump reflects the current machine
     if (gLocalByteOrder == EBO_LittleEndian)
         EXPECTED_DUMP += "# Used TransferSyntax: Little Endian Explicit\n";
     else
