@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2019-2022, Open Connections GmbH
+ *  Copyright (C) 2019-2024, Open Connections GmbH
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation are maintained by
@@ -23,10 +23,10 @@
 
 #include "dcmtk/dcmdata/dcdeftag.h"
 #include "dcmtk/dcmdata/dcitem.h"
-#include "dcmtk/dcmdata/dcpixel.h"
 #include "dcmtk/dcmdata/dcuid.h"
+#include "dcmtk/dcmdata/dcfilefo.h"
+#include "dcmtk/dcmiod/iodutil.h"
 #include "dcmtk/dcmfg/concatenationloader.h"
-#include "dcmtk/dcmfg/fg.h"
 #include "dcmtk/dcmfg/fgtypes.h"
 #include "dcmtk/ofstd/ofstd.h"
 
