@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2001-2022, OFFIS e.V.
+ *  Copyright (C) 2001-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -67,7 +67,8 @@ OFBool DJCodecEncoder::canChangeCoding(
   E_TransferSyntax myXfer = supportedTransferSyntax();
   DcmXfer newRep(newRepType);
   DcmXfer oldRep(oldRepType);
-  if (oldRep.isNotEncapsulated() && (newRepType == myXfer)) return OFTrue; // compress requested
+  if (oldRep.usesNativeFormat() && (newRepType == myXfer))
+    return OFTrue; // compress requested
 
   // we don't support re-coding for now
   return OFFalse;
@@ -1294,7 +1295,6 @@ OFCondition DJCodecEncoder::encodeMonochromeImage(
     delete dataset->remove(DCM_ModalityLUTSequence);
     delete dataset->remove(DCM_RescaleIntercept);
     delete dataset->remove(DCM_RescaleSlope);
-    delete dataset->remove(DCM_RescaleType);
 
     // update Modality LUT Module and Pixel Intensity Relationship
     if (windowType == 0)
@@ -1303,6 +1303,7 @@ OFCondition DJCodecEncoder::encodeMonochromeImage(
       {
         // XA Mode: set Pixel Intensity Relationship to "DISP", no Modality LUT
         if (result.good()) result = dataset->putAndInsertString(DCM_PixelIntensityRelationship, "DISP");
+        delete dataset->remove(DCM_RescaleType);
       }
       /* else if we had a modality LUT before, a LUT is inserted again.
          or if specific rescale slope/intercept has been computed, use that in image
@@ -1316,13 +1317,15 @@ OFCondition DJCodecEncoder::encodeMonochromeImage(
         if (result.good()) result = dataset->putAndInsertString(DCM_RescaleSlope, buf);
         if (result.good())
         {
-          if (mode_CT) result = dataset->putAndInsertString(DCM_RescaleType, "HU"); // Hounsfield units
-          else result =         dataset->putAndInsertString(DCM_RescaleType, "US"); // unspecified
+          // keep the old value of RescaleType for CT, set "US" (unspecified) otherwise
+          if (! mode_CT) dataset->putAndInsertString(DCM_RescaleType, "US");
         }
       }
     }
     else
     {
+      delete dataset->remove(DCM_RescaleType);
+
       // if we had found a Modality LUT Transformation, create a identity LUT transformation
       if (foundModalityLUT)
       {
@@ -1397,17 +1400,17 @@ OFCondition DJCodecEncoder::correctVOIWindows(
   DcmElement *explanation = NULL;
 
   DcmStack stack;
-  if ((dataset->search(DCM_WindowCenter, stack, ESM_fromHere, OFFalse)).good())
+  if ((dataset->search(DCM_WindowCenter, stack, ESM_fromHere, OFFalse)).good() && stack.top()->isElement())
   {
     center = OFreinterpret_cast(DcmElement*, stack.top());
   }
   stack.clear();
-  if ((dataset->search(DCM_WindowWidth, stack, ESM_fromHere, OFFalse)).good())
+  if ((dataset->search(DCM_WindowWidth, stack, ESM_fromHere, OFFalse)).good() && stack.top()->isElement())
   {
     width = OFreinterpret_cast(DcmElement*, stack.top());
   }
   stack.clear();
-  if ((dataset->search(DCM_WindowCenterWidthExplanation, stack, ESM_fromHere, OFFalse)).good())
+  if ((dataset->search(DCM_WindowCenterWidthExplanation, stack, ESM_fromHere, OFFalse)).good() && stack.top()->isElement())
   {
     explanation = OFreinterpret_cast(DcmElement*, stack.top());
   }
