@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2007-2022, OFFIS e.V.
+ *  Copyright (C) 2007-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -69,7 +69,7 @@ OFBool DJLSDecoderBase::canChangeCoding(
   // this codec only handles conversion from JPEG-LS to uncompressed.
 
   DcmXfer newRep(newRepType);
-  if (newRep.isNotEncapsulated() && (oldRepType == supportedTransferSyntax()))
+  if (newRep.usesNativeFormat() && (oldRepType == supportedTransferSyntax()))
      return OFTrue;
 
   return OFFalse;
@@ -143,8 +143,23 @@ OFCondition DJLSDecoderBase::decode(
   // compute size of uncompressed frame, in bytes
   Uint32 frameSize = bytesPerSample * imageRows * imageColumns * imageSamplesPerPixel;
 
+  // check for overflow
+  if (imageRows != 0 && frameSize / imageRows != (OFstatic_cast(Uint32, bytesPerSample) * imageColumns * imageSamplesPerPixel))
+  {
+    DCMJPLS_WARN("cannot decompress image because uncompressed representation would exceed maximum possible size of PixelData attribute");
+    return EC_ElemLengthExceeds32BitField;
+  }
+
   // compute size of pixel data attribute, in bytes
   Uint32 totalSize = frameSize * imageFrames;
+
+  // check for overflow
+  if (totalSize == 0xFFFFFFFF || (frameSize != 0 && totalSize / frameSize != OFstatic_cast(Uint32, imageFrames)))
+  {
+    DCMJPLS_WARN("cannot decompress image because uncompressed representation would exceed maximum possible size of PixelData attribute");
+    return EC_ElemLengthExceeds32BitField;
+  }
+
   if (totalSize & 1) totalSize++; // align on 16-bit word boundary
 
   // assume we can cast the codec parameter to what we need
@@ -196,7 +211,7 @@ OFCondition DJLSDecoderBase::decode(
   if (result.good() && (numberOfFramesPresent || (imageFrames > 1)))
   {
     char numBuf[20];
-    sprintf(numBuf, "%ld", OFstatic_cast(long, imageFrames));
+    OFStandard::snprintf(numBuf, sizeof(numBuf), "%ld", OFstatic_cast(long, imageFrames));
     result = ((DcmItem *)dataset)->putAndInsertString(DCM_NumberOfFrames, numBuf);
   }
 
@@ -633,7 +648,7 @@ Uint32 DJLSDecoderBase::computeNumberOfFragments(
 
   // So we have a multi-frame image with multiple fragments per frame and the
   // offset table is empty or wrong. Our last chance is to peek into the JPEG-LS
-  // bistream and identify the start of the next frame.
+  // bitstream and identify the start of the next frame.
   Uint32 nextItem = startItem;
   Uint8 *fragmentData = NULL;
   while (++nextItem < numItems)
