@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2021, OFFIS e.V.
+ *  Copyright (C) 1994-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were partly developed by
@@ -80,14 +80,6 @@
 
 #include "dcmtk/config/osconfig.h"    /* make sure OS specific configuration is included first */
 
-#ifdef HAVE_UNIX_H
-#if defined(macintosh) && defined (HAVE_WINSOCK_H)
-/* unix.h defines timeval incompatible with winsock.h */
-#define timeval _UNWANTED_timeval
-#endif
-#include <unix.h>       /* for unlink() under Metrowerks C++ (Macintosh) */
-#undef timeval
-#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -110,7 +102,7 @@
 #include "dcmtk/dcmdata/dcdicent.h"    /* for class DcmDictEntry, needed for MSVC5 */
 #include "dcmtk/dcmdata/dcwcache.h"    /* for class DcmWriteCache */
 #include "dcmtk/dcmdata/dcvrui.h"      /* for class DcmUniqueIdentifier */
-
+#include "dcmtk/ofstd/ofstd.h"
 
 /*
  * Global variables, mutex protected
@@ -181,13 +173,13 @@ static void saveDimseFragment(
   char filename[2048];
   if (isCommand)
   {
-    sprintf(filename, "dimse-cmd-%s-%04lu.dcm", transmission, g_dimse_commandCounter);
+    OFStandard::snprintf(filename, sizeof(filename), "dimse-cmd-%s-%04lu.dcm", transmission, g_dimse_commandCounter);
   } else {
     if (g_dimse_dataCounter < 2)
     {
-      sprintf(filename, "dimse-dat-%s-%04lu.dcm", transmission, g_dimse_commandCounter);
+      OFStandard::snprintf(filename, sizeof(filename), "dimse-dat-%s-%04lu.dcm", transmission, g_dimse_commandCounter);
     } else {
-      sprintf(filename, "dimse-dat-%s-%04lu-%02lu.dcm", transmission, g_dimse_commandCounter, g_dimse_dataCounter);
+      OFStandard::snprintf(filename, sizeof(filename), "dimse-dat-%s-%04lu-%02lu.dcm", transmission, g_dimse_commandCounter, g_dimse_dataCounter);
     }
   }
 
@@ -323,6 +315,7 @@ getTransferSyntax(
         case EXS_LittleEndianImplicit:
         case EXS_LittleEndianExplicit:
         case EXS_BigEndianExplicit:
+        case EXS_EncapsulatedUncompressedLittleEndianExplicit:
         case EXS_JPEGProcess1:
         case EXS_JPEGProcess2_4:
         case EXS_JPEGProcess3_5:
@@ -342,6 +335,9 @@ getTransferSyntax(
         case EXS_JPEGProcess29:
         case EXS_JPEGProcess14SV1:
         case EXS_RLELossless:
+#ifdef WITH_ZLIB
+        case EXS_DeflatedLittleEndianExplicit:
+#endif
         case EXS_JPEGLSLossless:
         case EXS_JPEGLSLossy:
         case EXS_JPEG2000LosslessOnly:
@@ -357,16 +353,26 @@ getTransferSyntax(
         case EXS_MPEG4StereoHighProfileLevel4_2:
         case EXS_HEVCMainProfileLevel5_1:
         case EXS_HEVCMain10ProfileLevel5_1:
-#ifdef WITH_ZLIB
-        case EXS_DeflatedLittleEndianExplicit:
-#endif
+        case EXS_FragmentableMPEG2MainProfileMainLevel:
+        case EXS_FragmentableMPEG2MainProfileHighLevel:
+        case EXS_FragmentableMPEG4HighProfileLevel4_1:
+        case EXS_FragmentableMPEG4BDcompatibleHighProfileLevel4_1:
+        case EXS_FragmentableMPEG4HighProfileLevel4_2_For2DVideo:
+        case EXS_FragmentableMPEG4HighProfileLevel4_2_For3DVideo:
+        case EXS_FragmentableMPEG4StereoHighProfileLevel4_2:
+        case EXS_HighThroughputJPEG2000LosslessOnly:
+        case EXS_HighThroughputJPEG2000withRPCLOptionsLosslessOnly:
+        case EXS_HighThroughputJPEG2000:
+        case EXS_JPEGXLLossless:
+        case EXS_JPEGXLJPEGRecompression:
+        case EXS_JPEGXL:
         /* OK, these can be supported */
         break;
     default:
         /* all other transfer syntaxes are not supported; hence, set the error indicator variable */
         {
           char buf[256];
-          sprintf(buf, "DIMSE Unsupported transfer syntax: %s", ts);
+          OFStandard::snprintf(buf, sizeof(buf), "DIMSE Unsupported transfer syntax: %s", ts);
           OFCondition subCond = makeDcmnetCondition(DIMSEC_UNSUPPORTEDTRANSFERSYNTAX, OF_error, buf);
           cond = makeDcmnetSubCondition(DIMSEC_RECEIVEFAILED, OF_error, "DIMSE Failed to receive message", subCond);
         }
@@ -1141,7 +1147,7 @@ DIMSE_receiveCommand(
         {
             delete cmdSet;
             char buf1[256];
-            sprintf(buf1, "DIMSE: Different PresIDs inside Command Set: %d != %d", pid, pdv.presentationContextID);
+            OFStandard::snprintf(buf1, sizeof(buf1), "DIMSE: Different PresIDs inside Command Set: %d != %d", pid, pdv.presentationContextID);
             OFCondition subCond = makeDcmnetCondition(DIMSEC_INVALIDPRESENTATIONCONTEXTID, OF_error, buf1);
             return makeDcmnetSubCondition(DIMSEC_RECEIVEFAILED, OF_error, "DIMSE Failed to receive message", subCond);
         }
@@ -1153,7 +1159,7 @@ DIMSE_receiveCommand(
         {
             /* This should NEVER happen.  See Part 7, Annex F. */
             char buf2[256];
-            sprintf(buf2, "DIMSE: Odd Fragment Length: %lu", pdv.fragmentLength);
+            OFStandard::snprintf(buf2, sizeof(buf2), "DIMSE: Odd Fragment Length: %lu", pdv.fragmentLength);
             cond = makeDcmnetCondition(DIMSEC_RECEIVEFAILED, OF_error, buf2);
             break;
         }
@@ -1443,7 +1449,7 @@ DIMSE_receiveDataSetInFile(
           else if (pdv.presentationContextID != pid)
           {
             char buf1[256];
-            sprintf(buf1, "DIMSE: Different PresIDs inside data set: %d != %d", pid, pdv.presentationContextID);
+            OFStandard::snprintf(buf1, sizeof(buf1), "DIMSE: Different PresIDs inside data set: %d != %d", pid, pdv.presentationContextID);
             OFCondition subCond = makeDcmnetCondition(DIMSEC_INVALIDPRESENTATIONCONTEXTID, OF_error, buf1);
             cond = makeDcmnetSubCondition(DIMSEC_RECEIVEFAILED, OF_error, "DIMSE Failed to receive message", subCond);
             last = OFTrue; // terminate loop
@@ -1456,7 +1462,7 @@ DIMSE_receiveDataSetInFile(
           {
             /* This should NEVER happen.  See Part 7, Annex F. */
             char buf2[256];
-            sprintf(buf2, "DIMSE: Odd Fragment Length: %lu", pdv.fragmentLength);
+            OFStandard::snprintf(buf2, sizeof(buf2), "DIMSE: Odd Fragment Length: %lu", pdv.fragmentLength);
             cond = makeDcmnetCondition(DIMSEC_RECEIVEFAILED, OF_error, buf2);
             last = OFTrue; // terminate loop
           }
@@ -1598,7 +1604,7 @@ DIMSE_receiveDataSetInMemory(
             else if (pdv.presentationContextID != pid)
             {
                 char buf1[256];
-                sprintf(buf1, "DIMSE: Different PresIDs inside data set: %d != %d", pid, pdv.presentationContextID);
+                OFStandard::snprintf(buf1, sizeof(buf1), "DIMSE: Different PresIDs inside data set: %d != %d", pid, pdv.presentationContextID);
                 OFCondition subCond = makeDcmnetCondition(DIMSEC_INVALIDPRESENTATIONCONTEXTID, OF_error, buf1);
                 cond = makeDcmnetSubCondition(DIMSEC_RECEIVEFAILED, OF_error, "DIMSE Failed to receive message", subCond);
                 last = OFTrue;
@@ -1613,7 +1619,7 @@ DIMSE_receiveDataSetInMemory(
             if ((pdv.fragmentLength % 2) != 0)
             {
                 char buf2[256];
-                sprintf(buf2, "DIMSE: Odd Fragment Length: %lu", pdv.fragmentLength);
+                OFStandard::snprintf(buf2, sizeof(buf2), "DIMSE: Odd Fragment Length: %lu", pdv.fragmentLength);
                 cond = makeDcmnetCondition(DIMSEC_RECEIVEFAILED, OF_error, buf2);
                 last = OFTrue;
             }
