@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1998-2021, OFFIS e.V.
+ *  Copyright (C) 1998-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -135,7 +135,7 @@ static OFCondition readTextFile(const char *filename,
                 {
                     while (file.get(c) && (c != '\n') && (c != '\r'));      // skip comments
                 }
-                else if (!isspace(OFstatic_cast(unsigned char, c)))         // skip whitespaces
+                else if (!OFStandard::isspace(c))                           // skip whitespaces
                 {
                     file.putback(c);
                     if (inputEntries == 0)                                  // read number of entries
@@ -284,7 +284,8 @@ static OFCondition convertInputLUT(const unsigned int numberOfBits,
                                    const unsigned int order,
                                    Uint16 *outputData,
                                    OFString &header,
-                                   char *explanation)
+                                   char *explanation,
+                                   size_t explanation_len)
 {
     OFCondition result = EC_IllegalCall;
     if ((inputXData != NULL) && (inputYData != NULL) && (inputEntries > 0) && (inputYMax > 0) && (outputData != NULL))
@@ -294,7 +295,7 @@ static OFCondition convertInputLUT(const unsigned int numberOfBits,
         {
             if (strlen(explanation) == 0)
             {
-                sprintf(explanation, "LUT created from %u->%u bit data, descriptor %u/%ld/%u",
+                OFStandard::snprintf(explanation, explanation_len, "LUT created from %u->%u bit data, descriptor %u/%ld/%u",
                     DicomImageClass::tobits((int)inputXMax), DicomImageClass::tobits((int)inputYMax),
                     (numberOfEntries < 65536) ? (Uint16)numberOfEntries : 0, firstMapped, numberOfBits);
             }
@@ -364,7 +365,8 @@ static void gammaLUT(const unsigned int numberOfBits,
                      const double gammaValue,
                      Uint16 *outputData,
                      OFString &header,
-                     char *explanation)
+                     char *explanation,
+                     size_t explanation_len)
 {
     if (outputData != NULL)
     {
@@ -376,8 +378,7 @@ static void gammaLUT(const unsigned int numberOfBits,
             {
                 char gammabuf[16];
                 OFStandard::ftoa(gammabuf, sizeof(gammabuf), gammaValue, OFStandard::ftoa_format_f, 3, 1);
-
-                sprintf(explanation, "LUT with gamma %s, descriptor %u/%ld/%u", gammabuf,
+                OFStandard::snprintf(explanation, explanation_len, "LUT with gamma %s, descriptor %u/%ld/%u", gammabuf,
                     (numberOfEntries < 65536) ? (Uint16)numberOfEntries : 0, firstMapped, numberOfBits);
             }
             oss << "# " << explanation << OFendl;
@@ -575,7 +576,7 @@ static OFCondition createLUT(const unsigned int numberOfBits,
     unsigned long wordsToWrite = 0;
     if (byteAlign)
     {
-        // the array is now in little endian byte order. Swap to local byte order if neccessary.
+        // the array is now in little endian byte order. Swap to local byte order if necessary.
         swapIfNecessary(gLocalByteOrder, EBO_LittleEndian, (Uint8 *)data, numberOfEntries + 1, sizeof(Uint16));
         wordsToWrite = (numberOfEntries + 1) / 2;
     } else
@@ -674,7 +675,7 @@ int main(int argc, char *argv[])
     cmd.setOptionColumns(LONGCOL, SHORTCOL);
     cmd.setParamColumn(LONGCOL + SHORTCOL + 4);
 
-    cmd.addParam("dcmimg-out",                   "DICOM output filename");
+    cmd.addParam("dcmimg-out",                   "DICOM output filename (\"-\" for stdout)");
 
     cmd.addGroup("general options:", LONGCOL, SHORTCOL + 2);
      cmd.addOption("--help",           "-h",     "print this help text and exit", OFCommandLine::AF_Exclusive);
@@ -893,7 +894,7 @@ int main(int argc, char *argv[])
         {
             char explStr[1024];
             if (opt_explanation != NULL)
-                OFStandard::strlcpy(explStr, opt_explanation, 1024);
+                OFStandard::strlcpy(explStr, opt_explanation, sizeof(explStr));
             else
                 explStr[0] = 0;
             OFString headerStr;
@@ -905,22 +906,22 @@ int main(int argc, char *argv[])
             if (readMapFile(opt_mapName, inputXData, inputYData, inputEntries, inputXMax, inputYMax) == EC_Normal)
             {
                 result = convertInputLUT((unsigned int)opt_bits, opt_entries, opt_firstMapped, inputXData, inputYData, inputEntries,
-                    inputXMax, inputYMax, (unsigned int)opt_order, outputData, headerStr, explStr);
+                    inputXMax, inputYMax, (unsigned int)opt_order, outputData, headerStr, explStr, sizeof(explStr));
             }
             else if (readTextFile(opt_textName, inputXData, inputYData, inputEntries, inputXMax, inputYMax) == EC_Normal)
             {
                 result = convertInputLUT((unsigned int)opt_bits, opt_entries, opt_firstMapped, inputXData, inputYData, inputEntries,
-                    inputXMax, inputYMax, (unsigned int)opt_order, outputData, headerStr, explStr);
+                    inputXMax, inputYMax, (unsigned int)opt_order, outputData, headerStr, explStr, sizeof(explStr));
             } else {
-                gammaLUT((unsigned int)opt_bits, opt_entries, opt_firstMapped, opt_byteAlign, opt_gammaValue, outputData, headerStr, explStr);
+                gammaLUT((unsigned int)opt_bits, opt_entries, opt_firstMapped, opt_byteAlign, opt_gammaValue, outputData, headerStr, explStr, sizeof(explStr));
             }
             if (result == EC_Normal)
             {
                 if (opt_inverseGSDF)
                     applyInverseGSDF((unsigned int)opt_bits, opt_entries, opt_byteAlign, (unsigned int)opt_minDensity, (unsigned int)opt_maxDensity,
-                        (unsigned int)opt_illumination, (unsigned int)opt_reflection, outputData, headerStr, explStr, 1024);
+                        (unsigned int)opt_illumination, (unsigned int)opt_reflection, outputData, headerStr, explStr, sizeof(explStr));
                 if (opt_randomCount > 0)
-                    mixingUpLUT(opt_entries, opt_byteAlign, opt_randomCount, (Uint32)opt_randomSeed, outputData, explStr, 1024);
+                    mixingUpLUT(opt_entries, opt_byteAlign, opt_randomCount, (Uint32)opt_randomSeed, outputData, explStr, sizeof(explStr));
                 result = createLUT((unsigned int)opt_bits, opt_entries, opt_firstMapped, opt_byteAlign, opt_lutVR, *ditem,
                     outputData, explStr);
             }
@@ -971,7 +972,7 @@ int main(int argc, char *argv[])
                     {
                         // search existing sequence
                         DcmStack stack;
-                        if (EC_Normal == dataset->search(DCM_PresentationLUTSequence, stack, ESM_fromHere, OFFalse))
+                        if (EC_Normal == dataset->search(DCM_PresentationLUTSequence, stack, ESM_fromHere, OFFalse) && (stack.top()->ident() == EVR_SQ))
                             dseq=(DcmSequenceOfItems *)stack.top();
                     }
                     if (dseq == NULL)
@@ -992,7 +993,7 @@ int main(int argc, char *argv[])
                     {
                         // search existing sequence
                         DcmStack stack;
-                        if (EC_Normal == dataset->search(DCM_VOILUTSequence, stack, ESM_fromHere, OFFalse))
+                        if (EC_Normal == dataset->search(DCM_VOILUTSequence, stack, ESM_fromHere, OFFalse) && (stack.top()->ident() == EVR_SQ))
                             dseq=(DcmSequenceOfItems *)stack.top();
                     }
                     if (dseq == NULL)
