@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1999-2021, OFFIS e.V.
+ *  Copyright (C) 1999-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -417,6 +417,7 @@ int main(int argc, char *argv[])
     const char *targetDescription = dvi.getTargetDescription(opt_target);
     const char *targetAETitle     = dvi.getTargetAETitle(opt_target);
     unsigned short targetPort     = dvi.getTargetPort(opt_target);
+    T_ASC_ProtocolFamily targetProtocol = dvi.getTargetProtocol(opt_target);
     unsigned long  targetMaxPDU   = dvi.getTargetMaxPDU(opt_target);
     OFBool targetImplicitOnly     = dvi.getTargetImplicitOnly(opt_target);
     OFBool targetDisableNewVRs    = dvi.getTargetDisableNewVRs(opt_target);
@@ -543,7 +544,12 @@ int main(int argc, char *argv[])
     else if (targetImplicitOnly) verboseParameters << "implicit xfer syntax only";
     else if (targetDisableNewVRs) verboseParameters << "disable post-1993 VRs";
     else verboseParameters << "none";
-    verboseParameters << OFendl;
+    verboseParameters << OFendl << "\tprotocol version: ";
+
+    if (targetProtocol == ASC_AF_INET) verboseParameters << "IPv4 only" << OFendl;
+    else if (targetProtocol == ASC_AF_INET6) verboseParameters << "IPv6 only" << OFendl;
+    else if (targetProtocol == ASC_AF_UNSPEC) verboseParameters << "determined via DNS" << OFendl;
+    else verboseParameters << "default" << OFendl;
 
     verboseParameters << "\tTLS             : ";
     if (useTLS) verboseParameters << "enabled" << OFendl; else verboseParameters << "disabled" << OFendl;
@@ -577,8 +583,10 @@ int main(int argc, char *argv[])
       OFString profileName;
       const char *profileNamePtr = dvi.getTargetTLSProfile(opt_target);
       if (profileNamePtr) profileName = profileNamePtr;
-      DcmTLSSecurityProfile tlsProfile = TSP_Profile_BCP195;  // default
-      if (profileName == "BCP195") tlsProfile = TSP_Profile_BCP195;
+      DcmTLSSecurityProfile tlsProfile = TSP_Profile_BCP_195_RFC_8996;  // default
+      if (profileName == "BCP195-RFC8996") tlsProfile = TSP_Profile_BCP_195_RFC_8996;
+      else if (profileName == "BCP195-RFC8996-MOD") tlsProfile = TSP_Profile_BCP_195_RFC_8996_Modified;
+      else if (profileName == "BCP195") tlsProfile = TSP_Profile_BCP195;
       else if (profileName == "BCP195-ND") tlsProfile = TSP_Profile_BCP195_ND;
       else if (profileName == "BCP195-EX") tlsProfile = TSP_Profile_BCP195_Extended;
       else if (profileName == "AES") tlsProfile = TSP_Profile_AES;
@@ -619,7 +627,7 @@ int main(int argc, char *argv[])
           OFLOG_FATAL(dcmpssndLogger, "unable to load private TLS key from '" << tlsPrivateKeyFile<< "'");
           return 1;
         }
-        if (tLayer->setCertificateFile(tlsCertificateFile.c_str(), keyFileFormat).bad())
+        if (tLayer->setCertificateFile(tlsCertificateFile.c_str(), keyFileFormat, tlsProfile).bad())
         {
           OFLOG_FATAL(dcmpssndLogger, "unable to load certificate from '" << tlsCertificateFile << "'");
           return 1;
@@ -697,7 +705,7 @@ int main(int argc, char *argv[])
     }
 #endif
 
-    cond = ASC_createAssociationParameters(&params, targetMaxPDU);
+    cond = ASC_createAssociationParameters(&params, targetMaxPDU, dcmConnectionTimeout.get());
     if (cond.bad())
     {
       OFLOG_FATAL(dcmpssndLogger, DimseCondition::dump(temp_str, cond));
@@ -712,8 +720,9 @@ int main(int argc, char *argv[])
     }
 
     ASC_setAPTitles(params, dvi.getNetworkAETitle(), targetAETitle, NULL);
+    ASC_setProtocolFamily(params, targetProtocol);
 
-    sprintf(peerHost, "%s:%d", targetHostname, (int)targetPort);
+    OFStandard::snprintf(peerHost, sizeof(peerHost), "%s:%d", targetHostname, (int)targetPort);
     ASC_setPresentationAddresses(params, OFStandard::getHostName().c_str(), peerHost);
 
     cond = addAllStoragePresentationContexts(params, targetImplicitOnly);
