@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1993-2021, OFFIS e.V.
+ *  Copyright (C) 1993-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -19,7 +19,7 @@
  *
  */
 
-#include "dcmtk/config/osconfig.h"    /* make sure OS specific configuration is included first */
+#include "dcmtk/config/osconfig.h"     /* make sure OS specific configuration is included first */
 #include "dcmtk/dcmqrdb/dcmqrsrv.h"
 #include "dcmtk/dcmqrdb/dcmqropt.h"
 #include "dcmtk/dcmdata/dcfilefo.h"
@@ -30,7 +30,7 @@
 #include "dcmtk/dcmqrdb/dcmqrcbm.h"    /* for class DcmQueryRetrieveMoveContext */
 #include "dcmtk/dcmqrdb/dcmqrcbg.h"    /* for class DcmQueryRetrieveGetContext */
 #include "dcmtk/dcmqrdb/dcmqrcbs.h"    /* for class DcmQueryRetrieveStoreContext */
-
+#include "dcmtk/dcmtls/tlsopt.h"       /* for DcmTLSOptions */
 
 static void findCallback(
   /* in */
@@ -100,7 +100,8 @@ DcmQueryRetrieveSCP::DcmQueryRetrieveSCP(
   const DcmQueryRetrieveConfig& config,
   const DcmQueryRetrieveOptions& options,
   const DcmQueryRetrieveDatabaseHandleFactory& factory,
-  const DcmAssociationConfiguration& associationConfiguration)
+  const DcmAssociationConfiguration& associationConfiguration,
+  DcmTLSOptions& tlsOptions)
 : config_(&config)
 , processtable_()
 , dbCheckFindIdentifier_(OFFalse)
@@ -108,6 +109,7 @@ DcmQueryRetrieveSCP::DcmQueryRetrieveSCP(
 , factory_(factory)
 , options_(options)
 , associationConfiguration_(associationConfiguration)
+, tlsOptions_(tlsOptions)
 {
 }
 
@@ -205,13 +207,19 @@ OFCondition DcmQueryRetrieveSCP::dispatch(T_ASC_Association *assoc, OFBool corre
 }
 
 
-OFCondition DcmQueryRetrieveSCP::handleAssociation(T_ASC_Association * assoc, OFBool correctUIDPadding)
+OFCondition DcmQueryRetrieveSCP::handleAssociation(T_ASC_Association ** pAssoc, OFBool correctUIDPadding)
 {
     OFCondition         cond = EC_Normal;
     DIC_NODENAME        peerHostName;
     DIC_AE              peerAETitle;
     DIC_AE              myAETitle;
     OFString            temp_str;
+    T_ASC_Association * assoc;
+
+    if (pAssoc == NULL) {
+        return EC_IllegalParameter;
+    }
+    assoc = *pAssoc;
 
     ASC_getPresentationAddresses(assoc->params, peerHostName, sizeof(peerHostName), NULL, 0);
     ASC_getAPTitles(assoc->params, peerAETitle, sizeof(peerAETitle), myAETitle, sizeof(myAETitle), NULL, 0);
@@ -236,7 +244,7 @@ OFCondition DcmQueryRetrieveSCP::handleAssociation(T_ASC_Association * assoc, OF
     if (cond.bad()) {
         DCMQRDB_ERROR("Cannot Drop Association: " << DimseCondition::dump(temp_str, cond));
     }
-    cond = ASC_destroyAssociation(&assoc);
+    cond = ASC_destroyAssociation(pAssoc);
     if (cond.bad()) {
         DCMQRDB_ERROR("Cannot Destroy Association: " << DimseCondition::dump(temp_str, cond));
     }
@@ -994,7 +1002,7 @@ OFCondition DcmQueryRetrieveSCP::waitForAssociation(T_ASC_Network * theNet)
 
     if (ASC_associationWaiting(theNet, timeout))
     {
-        cond = ASC_receiveAssociation(theNet, &assoc, (int)options_.maxPDU_);
+        cond = ASC_receiveAssociation(theNet, &assoc, (int)options_.maxPDU_, NULL, NULL, tlsOptions_.secureConnectionRequested());
         if (cond.bad())
         {
           DCMQRDB_INFO("Failed to receive association: " << DimseCondition::dump(temp_str, cond));
@@ -1096,7 +1104,7 @@ OFCondition DcmQueryRetrieveSCP::waitForAssociation(T_ASC_Network * theNet)
         if (options_.singleProcess_)
         {
             /* don't spawn a sub-process to handle the association */
-            cond = handleAssociation(assoc, options_.correctUIDPadding_);
+            cond = handleAssociation(&assoc, options_.correctUIDPadding_);
         }
 #ifdef HAVE_FORK
         else
@@ -1112,13 +1120,19 @@ OFCondition DcmQueryRetrieveSCP::waitForAssociation(T_ASC_Network * theNet)
             }
             else if (pid > 0)
             {
-                /* parent process, note process in table */
+                // parent process, note process in table and close network connection.
+
+                // set the parent process mode in the association.
+                // This will prevent the TLS connection to be shutdown by ASC_dropAssociation().
+                ASC_setParentProcessMode(assoc);
+
+                // note process in table
                 processtable_.addProcessToTable(pid, assoc);
             }
             else
             {
                 /* child process, handle the association */
-                cond = handleAssociation(assoc, options_.correctUIDPadding_);
+                cond = handleAssociation(&assoc, options_.correctUIDPadding_);
                 /* the child process is done so exit */
                 exit(0);
             }
@@ -1126,24 +1140,22 @@ OFCondition DcmQueryRetrieveSCP::waitForAssociation(T_ASC_Network * theNet)
 #endif
     }
 
-    // cleanup code
-    OFCondition oldcond = cond;    /* store condition flag for later use */
-    if (!options_.singleProcess_ && (cond != ASC_SHUTDOWNAPPLICATION))
+    // avoid double deletion of memory
+    if (cond != ASC_SHUTDOWNAPPLICATION)
     {
-        /* the child will handle the association, we can drop it */
-        cond = ASC_dropAssociation(assoc);
-        if (cond.bad())
-        {
-            DCMQRDB_ERROR("Cannot Drop Association: " << DimseCondition::dump(temp_str, cond));
-        }
-        cond = ASC_destroyAssociation(&assoc);
-        if (cond.bad())
-        {
-            DCMQRDB_ERROR("Cannot Destroy Association: " << DimseCondition::dump(temp_str, cond));
-        }
+      // clean-up association
+      cond = ASC_dropAssociation(assoc);
+      if (cond.bad())
+      {
+          DCMQRDB_ERROR("Cannot Drop Association: " << DimseCondition::dump(temp_str, cond));
+      }
+      cond = ASC_destroyAssociation(&assoc);
+      if (cond.bad())
+      {
+          DCMQRDB_ERROR("Cannot Destroy Association: " << DimseCondition::dump(temp_str, cond));
+      }
     }
 
-    if (oldcond == ASC_SHUTDOWNAPPLICATION) cond = oldcond; /* abort flag is reported to top-level wait loop */
     return cond;
 }
 
