@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2015-2022, OFFIS e.V.
+ *  Copyright (C) 2015-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -24,7 +24,10 @@
 #include "dcmtk/ofstd/oftest.h"
 
 OFTEST_REGISTER(dcmseg_bigdim);
-OFTEST_REGISTER(dcmseg_utils);
+OFTEST_REGISTER(dcmseg_debugByte2Bin);
+OFTEST_REGISTER(dcmseg_packBinaryFrame);
+OFTEST_REGISTER(dcmseg_packAndUnpackBinaryFrame);
+OFTEST_REGISTER(dcmseg_concatBinaryFrames);
 OFTEST_REGISTER(dcmseg_roundtrip);
 OFTEST_REGISTER(dcmseg_concat_binary);
 
