@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2000-2019, OFFIS e.V.
+ *  Copyright (C) 2000-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -79,24 +79,6 @@ DSRDocumentTreeNode::~DSRDocumentTreeNode()
 }
 
 
-OFBool DSRDocumentTreeNode::operator==(const DSRDocumentTreeNode &node) const
-{
-    /* only very basic information is used for comparing the two nodes */
-    return (RelationshipType == node.RelationshipType) &&
-           (ValueType == node.ValueType) &&
-           (ConceptName == node.ConceptName);
-}
-
-
-OFBool DSRDocumentTreeNode::operator!=(const DSRDocumentTreeNode &node) const
-{
-    /* only very basic information is used for comparing the two nodes */
-    return (RelationshipType != node.RelationshipType) ||
-           (ValueType != node.ValueType) ||
-           (ConceptName != node.ConceptName);
-}
-
-
 void DSRDocumentTreeNode::clear()
 {
     MarkFlag = OFFalse;
@@ -112,6 +94,24 @@ void DSRDocumentTreeNode::clear()
 }
 
 
+OFBool DSRDocumentTreeNode::isEqual(const DSRDocumentTreeNode &node) const
+{
+    /* only very basic information is used for comparing the two nodes */
+    return (RelationshipType == node.RelationshipType) &&
+           (ValueType == node.ValueType) &&
+           (ConceptName == node.ConceptName);
+}
+
+
+OFBool DSRDocumentTreeNode::isNotEqual(const DSRDocumentTreeNode &node) const
+{
+    /* only very basic information is used for comparing the two nodes */
+    return (RelationshipType != node.RelationshipType) ||
+           (ValueType != node.ValueType) ||
+           (ConceptName != node.ConceptName);
+}
+
+
 OFBool DSRDocumentTreeNode::isValid() const
 {
     return (RelationshipType != RT_invalid) && (ValueType != VT_invalid);
@@ -149,7 +149,7 @@ OFCondition DSRDocumentTreeNode::print(STD_NAMESPACE ostream &stream,
     stream << valueTypeToDefinedTerm(ValueType);
     DCMSR_PRINT_ANSI_ESCAPE_CODE(DCMSR_ANSI_ESCAPE_CODE_DELIMITER)
     stream << ":";
-    /* only print valid concept name codes */
+    /* only print valid concept name codes (unless explicitly requested otherwise) */
     if (ConceptName.isValid() || (flags & PF_printInvalidCodes))
     {
         DCMSR_PRINT_ANSI_ESCAPE_CODE(DCMSR_ANSI_ESCAPE_CODE_CONCEPT_NAME)
@@ -453,7 +453,8 @@ OFCondition DSRDocumentTreeNode::renderHTML(STD_NAMESPACE ostream &docStream,
                                             STD_NAMESPACE ostream &annexStream,
                                             const size_t nestingLevel,
                                             size_t &annexNumber,
-                                            const size_t flags) const
+                                            const size_t flags,
+                                            const char *urlPrefix) const
 {
     /* check for validity */
     if (!isValid())
@@ -466,10 +467,10 @@ OFCondition DSRDocumentTreeNode::renderHTML(STD_NAMESPACE ostream &docStream,
         docStream << "<a " << attrName << "=\"content_item_" << getNodeID() << "\"" << closeElm << ">" << OFendl;
     }
     /* render content item */
-    OFCondition result = renderHTMLContentItem(docStream, annexStream, nestingLevel, annexNumber, flags);
+    OFCondition result = renderHTMLContentItem(docStream, annexStream, nestingLevel, annexNumber, flags, urlPrefix);
     /* render child nodes */
     if (result.good())
-        result = renderHTMLChildNodes(docStream, annexStream, nestingLevel, annexNumber, flags | HF_renderItemsSeparately);
+        result = renderHTMLChildNodes(docStream, annexStream, nestingLevel, annexNumber, flags | HF_renderItemsSeparately, urlPrefix);
     else
         printContentItemErrorMessage("Rendering", result, this);
     return result;
@@ -674,6 +675,18 @@ OFCondition DSRDocumentTreeNode::renderHTMLContentItem(STD_NAMESPACE ostream & /
 }
 
 
+OFCondition DSRDocumentTreeNode::renderHTMLContentItem(STD_NAMESPACE ostream &docStream,
+                                                       STD_NAMESPACE ostream &annexStream,
+                                                       const size_t nestingLevel,
+                                                       size_t &annexNumber,
+                                                       const size_t flags,
+                                                       const char * /*urlPrefix*/) const
+{
+    /* call the real method (without "urlPrefix" parameter) */
+    return renderHTMLContentItem(docStream, annexStream, nestingLevel, annexNumber, flags);
+}
+
+
 OFCondition DSRDocumentTreeNode::readSRDocumentContentModule(DcmItem &dataset,
                                                              const DSRIODConstraintChecker *constraintChecker,
                                                              const size_t flags)
@@ -971,7 +984,7 @@ OFCondition DSRDocumentTreeNode::readContentSequence(DcmItem &dataset,
             OFString location = posString;
             if (!location.empty())
                 location += ".";
-            location += numberToString(OFstatic_cast(size_t, i + 1), buffer);
+            location += numberToString(OFstatic_cast(size_t, i + 1), buffer, sizeof(buffer));
             if (flags & RF_showCurrentlyProcessedItem)
                 DCMSR_INFO("Processing content item " << location);
             /* read RelationshipType */
@@ -1169,7 +1182,8 @@ OFCondition DSRDocumentTreeNode::renderHTMLChildNodes(STD_NAMESPACE ostream &doc
                                                       STD_NAMESPACE ostream &annexStream,
                                                       const size_t nestingLevel,
                                                       size_t &annexNumber,
-                                                      const size_t flags) const
+                                                      const size_t flags,
+                                                      const char *urlPrefix) const
 {
     OFCondition result = EC_Normal;
     /* goto first child of current node */
@@ -1235,7 +1249,7 @@ OFCondition DSRDocumentTreeNode::renderHTMLChildNodes(STD_NAMESPACE ostream &doc
                         docStream << " = ";
                     }
                     /* render HTML code (directly to the reference text) */
-                    result = node->renderHTML(docStream, annexStream, 0 /*nesting level*/, annexNumber, newFlags | HF_renderItemInline);
+                    result = node->renderHTML(docStream, annexStream, 0 /*nesting level*/, annexNumber, newFlags | HF_renderItemInline, urlPrefix);
                 } else {
                     /* render concept name or value type */
                     if (node->getConceptName().getCodeMeaning().empty())
@@ -1251,7 +1265,7 @@ OFCondition DSRDocumentTreeNode::renderHTMLChildNodes(STD_NAMESPACE ostream &doc
                     /* create memory output stream for the temporal annex */
                     OFOStringStream tempAnnexStream;
                     /* render HTML code (directly to the annex) */
-                    result = node->renderHTML(annexStream, tempAnnexStream, 0 /*nesting level*/, annexNumber, newFlags | HF_currentlyInsideAnnex);
+                    result = node->renderHTML(annexStream, tempAnnexStream, 0 /*nesting level*/, annexNumber, newFlags | HF_currentlyInsideAnnex, urlPrefix);
                     annexStream << "</div>" << OFendl;
                     /* use empty paragraph for bottom margin */
                     if (!(flags & HF_XHTML11Compatibility))
@@ -1286,7 +1300,7 @@ OFCondition DSRDocumentTreeNode::renderHTMLChildNodes(STD_NAMESPACE ostream &doc
                 if (newFlags & HF_createFootnoteReferences)
                 {
                     /* render HTML code (without child nodes) */
-                    result = node->renderHTMLContentItem(docStream, annexStream, 0 /*nestingLevel*/, annexNumber, newFlags);
+                    result = node->renderHTMLContentItem(docStream, annexStream, 0 /*nestingLevel*/, annexNumber, newFlags, urlPrefix);
                     /* create footnote numbers (individually for each child?) */
                     if (result.good())
                     {
@@ -1302,11 +1316,11 @@ OFCondition DSRDocumentTreeNode::renderHTMLChildNodes(STD_NAMESPACE ostream &doc
                         /* render footnote text and reference */
                         createHTMLFootnote(docStream, tempDocStream, footnoteNumber, node->getNodeID(), flags);
                         /* render child nodes to temporary stream */
-                        result = node->renderHTMLChildNodes(tempDocStream, annexStream, 0 /*nestingLevel*/, annexNumber, newFlags);
+                        result = node->renderHTMLChildNodes(tempDocStream, annexStream, 0 /*nestingLevel*/, annexNumber, newFlags, urlPrefix);
                     }
                 } else {
                     /* render HTML code (incl. child nodes)*/
-                    result = node->renderHTML(docStream, annexStream, nestingLevel + 1, annexNumber, newFlags);
+                    result = node->renderHTML(docStream, annexStream, nestingLevel + 1, annexNumber, newFlags, urlPrefix);
                 }
                 /* end paragraph */
                 if (flags & HF_renderItemsSeparately)
@@ -1390,3 +1404,19 @@ OFBool DSRDocumentTreeNode::checkTemplateIdentification(const OFString &template
         result = OFTrue;
     return result;
 }
+
+
+// comparison operators
+
+OFBool operator==(const DSRDocumentTreeNode &lhs,
+                  const DSRDocumentTreeNode &rhs)
+{
+    return lhs.isEqual(rhs);
+}
+
+
+OFBool operator!=(const DSRDocumentTreeNode &lhs,
+                  const DSRDocumentTreeNode &rhs)
+{
+    return lhs.isNotEqual(rhs);
+}
