@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2002-2017, OFFIS e.V.
+ *  Copyright (C) 2002-2021, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -22,18 +22,15 @@
 
 #include "dcmtk/config/osconfig.h"
 
-#define INCLUDE_CTIME
-#define INCLUDE_CCTYPE
-#define INCLUDE_CSTRING
-#include "dcmtk/ofstd/ofstdinc.h"
-
-BEGIN_EXTERN_C
 #ifdef HAVE_SYS_TYPES_H
+BEGIN_EXTERN_C
 #include <sys/types.h>    /* for struct time_t */
-#endif
 END_EXTERN_C
+#endif
 
 #include "dcmtk/ofstd/ofdatime.h"
+#include "dcmtk/ofstd/ofstdinc.h"
+#include <ctime>
 
 
 /*------------------*
